<?php

class widget {

    protected $_isset = false;
    protected $_data = array();

    function __construct($path) {


        $this->_data ['path_abs'] = realpath($path);
        $this->_data ['runame'] = $this->_data ['name'] = basename($this->_data ['path_abs']);
        $this->_data ['version'] = '1.0'; // версия
        $this->_data ['autor'] = false; // автор
        $this->_data ['script'] = 'index.php'; // исполняемый скрипт
        $this->_data ['screen'] = false; // скриншот
        $this->_data ['cache_by_language'] = true; // отдельный кэш для каждого языка
        $this->_data ['cache_by_user'] = false; // отдельный кэш для каждого пользователя
        $this->_data ['cache_by_group'] = false; // отдельный кэш для каждой группы
        $this->_data ['cache_by_browser_type'] = false; // отдельный кэш для каждого типа браузера
        $this->_data ['cache_time'] = rand(10, 30); // время хранения кэша в секундах
        if ($config = ini::read($this->_data ['path_abs'] . '/config.ini')) {
            $this->_isset = true;
            // загружаем конфиг
            $this->_data = array_merge($this->_data, (array) @$config);
        }
    }

    // отображаем виджет
    function display() {
        if (!$this->_isset) {
            return false;
        }
        if ($content = $this->getContent()) {
            $widget = new design ();
            $widget->assign('content', $content);
            $widget->assign('name', $this->_data ['runame']);
            $widget->display('widget.tpl');
        }
    }

    // получаем содержимое виджета
    function getContent() {
        if (!$this->_isset) {
            return false;
        }

        global $user, $dcms;
        if ($cache_content = cache_widgets::get($this->_getCacheId())) {
            return $cache_content;
        }

        ob_start();
        include $this->_data ['path_abs'] . '/' . $this->_data ['script'];
        $content = ob_get_contents();
        ob_end_clean();

        cache_widgets::set($this->_getCacheId(), $content, mt_rand($this->_data ['cache_time'] - 1, $this->_data ['cache_time'] + 1));
        return $content;
    }

    // получаем уникальный идентификатор в кэше
    protected function _getCacheId() {
        if (!$this->_isset) {
            return false;
        }
        global $user, $dcms, $user_language_pack;
        $cache_id = array();
        $cache_id [] = 'widget-' . $this->_data ['name'];

        $cache_id [] = 'language-' . $user_language_pack->code;

        if ($this->_data ['cache_by_browser_type']) {
            $cache_id [] = 'browser-' . $dcms->browser_type;
        }

        if ($this->_data ['cache_by_user']) {
            $cache_id [] = 'user-' . $user->id;
        }

        if ($this->_data ['cache_by_group']) {
            $cache_id [] = 'group-' . intval($user->group);
        }

        if (SID) {
            // если браузер не поддерживает cookie, то во все ссылки будет добавляться SID,
            // поэтому кэш делаем для каждой сессии свой
            $cache_id [] = 'session-' . SID;
        }


        return implode('.', $cache_id);
    }

    function __get($n) {
        if (!$this->_isset) {
            return false;
        }

        return isset($this->_data [$n]) ? $this->_data [$n] : false;
    }

    function __set($n, $v) {
        if (!$this->_isset) {
            return false;
        }


        if (!isset($this->_data [$n])) {
            return false;
        }
        $this->_data [$n] = $v;
    }

    function saveData() {
        if (!$this->_isset) {
            return false;
        }


        return ini::save($this->_data ['path_abs'] . '/config.ini', $this->_data);
    }

}

?>
